#!/usr/bin/env python3
import argparse
import sys
from mge_masker.mge_masker_functions import get_mge_patterns, find_mges, create_mge_gff_file

class EnhancedParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)

def parse_args():
    parser = EnhancedParser()
    parser.add_argument('-i', '--genome_file_path', help='path to a genome file', required=True)
    parser.add_argument('-f', '--file_format', help='genome file format', default = "genbank")
    parser.add_argument('-m', '--mge_file_path', help='path to a file containing regex MGE annotations')
    return(parser.parse_args())


def main():
    options = parse_args()
    mges = find_mges(options.genome_file_path, options.file_format, options.mge_file_path)
    gff_file_path = create_mge_gff_file(options.genome_file_path, options.file_format, mges)
    print(f'GFF file written to {gff_file_path}')
    
 

if __name__ == "__main__":
    main()




from Bio import SeqIO
import inspect
import sys
import os
import re
from pathlib import Path

def get_mge_patterns(mge_file_path = None):
  # read text file with pattern match regular expressions and return a list of compiled regexs
  if not mge_file_path:
    mge_file_path = os.path.join(os.path.dirname(__file__), "mge_patterns.txt")
  mge_patterns = []
  with open(mge_file_path) as mge_file:
    for line in mge_file.readlines():
      pattern = re.compile(line.rstrip())
      mge_patterns.append(pattern)
  return(mge_patterns)

def get_features(genome_file_path, file_format):
  record = SeqIO.read(genome_file_path, file_format)
  return(record.features)

def create_gff_line(accession, mge):
  return(f'{accession}\t.\t{mge["type"]}\t{mge["start"]}\t{mge["end"]}\t.\t{mge["strand"]}\t.\tproduct="{mge["product"]}"\n')

def search_features_for_patterns(features, mge_patterns):
  mges = []
  for feature in features:
    products = feature.qualifiers.get("product")
    if products:
      for product in products:
        for pattern in mge_patterns:
          if pattern.match(product):
            if feature.strand:
              if feature.strand == 1:
                strand = "+"
              elif feature.strand == -1:
                strand = "-"
            else:
              strand = "."
            mges.append(
              {
                "type": feature.type,
                "start": feature.location.start.position + 1,
                "end": feature.location.end.position,
                "strand": strand,
                "product": product
              }
            )
  return(mges)

def find_mges(genome_file_path, file_format, mge_file_path = None):
  features = get_features(genome_file_path, file_format)
  mge_patterns = get_mge_patterns(mge_file_path)
  mges = search_features_for_patterns(features, mge_patterns)
  return(mges)

def create_mge_gff_file(genome_file_path, file_format, mges):
  record = SeqIO.read(genome_file_path, file_format)
  gff_file_path = Path(genome_file_path).with_suffix('.gff')
  with open(gff_file_path, "w") as gff_file:
    for mge in mges:
      gff_file.write(create_gff_line(record.name, mge))
  return(gff_file_path)
  


  


import pytest
import os
import inspect
import tempfile
from mge_masker.mge_masker_functions import find_mges, create_mge_gff_file

@pytest.fixture(scope="module")
def test_genome_file_path():
  test_genome_file_path = os.path.realpath(
    os.path.abspath(
      os.path.join(
        os.path.split(inspect.getfile( inspect.currentframe() ))[0],
        "data",
        "test.gbk"
      )
    )
  )
  return(test_genome_file_path)

@pytest.fixture(scope="module")
def expected_gff_file_path():
  expected_gff_file_path = os.path.realpath(
    os.path.abspath(
      os.path.join(
        os.path.split(inspect.getfile( inspect.currentframe() ))[0],
        "data",
        "test.gff"
      )
    )
  )
  return(expected_gff_file_path)

@pytest.fixture(scope="module")
def expected_mges():
  return (
    [
      {'type': 'CDS', 'start': 201, 'end': 400, 'strand': '+', 'product': 'Transposon'},
      {'type': 'CDS', 'start': 401, 'end': 500, 'strand': '-', 'product': 'Phage protein'},
      {'type': 'CDS', 'start': 501, 'end': 600, 'strand': '+', 'product': 'IS101'},
      {'type': 'CDS', 'start': 601, 'end': 700, 'strand': '+', 'product': 'Integrase'}
    ]
  )
def test_find_mges(test_genome_file_path, expected_mges):
  mges = find_mges(test_genome_file_path, "genbank")
  
  for index, mge in enumerate(mges):
    assert mge == expected_mges[index]
    
def test_create_file(test_genome_file_path, expected_gff_file_path):
  with tempfile.NamedTemporaryFile(mode = "w") as temp_genome:
    with open(test_genome_file_path) as test_genome_file:
      genome_string = test_genome_file.read()
    temp_genome.write(genome_string)
    temp_genome.flush()
    mges = find_mges(temp_genome.name, "genbank")
    gff_file_path = create_mge_gff_file(temp_genome.name, "genbank", mges)
    with open(gff_file_path) as gff_file:
      gff_string = gff_file.read()
    with open(expected_gff_file_path) as expected_gff_file:
      expected_gff_string = expected_gff_file.read()
    assert gff_string == expected_gff_string
      
